@@ -25,12 +25,14 @@
 
 #include <qplatformdefs.h>
 
+#include <QCoreApplication>
 #include <QAction>
 #include <QtCore/QMap>
 #include <QtCore/QSet>
 #include <QtCore/QFileInfo>
 #include <QtCore/QDir>
 #include <QDebug>
+#include <QResource>
 
 #include <klocalizedstring.h> // KLocalizedString::localizedFilePath. Need such functionality in, hmm, QLocale? QStandardPaths?
 
@@ -38,11 +40,40 @@
 
 #include <kconfiggroup.h>
 
+Q_GLOBAL_STATIC(QString, _themeOverride)
+
+// Support for icon themes in RCC files.
+// The intended use case is standalone apps on Windows / MacOS / etc.
+// For this reason we use AppDataLocation: BINDIR/data on Windows, Resources on OS X
+void initRCCIconTheme()
+{
+    const QString iconThemeRcc = QStandardPaths::locate(QStandardPaths::AppDataLocation, QStringLiteral("icontheme.rcc"));
+    if (!iconThemeRcc.isEmpty()) {
+        const QString iconThemeName = QStringLiteral("kf5_rcc_theme");
+        const QString iconSubdir = QStringLiteral("/icons/") + iconThemeName;
+        if (QResource::registerResource(iconThemeRcc, iconSubdir)) {
+            if (QFileInfo::exists(QLatin1Char(':') + iconSubdir + QStringLiteral("/index.theme"))) {
+                // Tell Qt about the theme
+                // Note that since qtbase commit a8621a3f8, this means the QPA (i.e. KIconLoader) will NOT be used.
+                QIcon::setThemeName(iconThemeName); // Qt looks under :/icons automatically
+                // Tell KIconTheme about the theme, in case KIconLoader is used directly
+                *_themeOverride() = iconThemeName;
+            } else {
+                qWarning() << "No index.theme found in" << iconThemeRcc;
+                QResource::unregisterResource(iconThemeRcc, iconSubdir);
+            }
+        } else {
+            qWarning() << "Invalid rcc file" << iconThemeRcc;
+        }
+    }
+}
+Q_COREAPP_STARTUP_FUNCTION(initRCCIconTheme)
+
+
 class KIconTheme::KIconThemePrivate
 {
 public:
     QString example, screenshot;
-    QString linkOverlay, lockOverlay, zipOverlay, shareOverlay;
     bool hidden;
     KSharedConfig::Ptr sharedConfig;
 
@@ -56,7 +87,6 @@ public:
     QVector<KIconThemeDir *> mDirs;
     bool followsColorScheme : 1;
 };
-Q_GLOBAL_STATIC(QString, _themeOverride)
 Q_GLOBAL_STATIC(QString, _theme)
 Q_GLOBAL_STATIC(QStringList, _theme_list)
 
@@ -134,7 +164,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
         const QStringList icnlibs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
         for (QStringList::ConstIterator it = icnlibs.constBegin(); it != icnlibs.constEnd(); ++it) {
             const QString cDir = *it + '/' + appName + "/icons/" + name + '/';
-            if (QFile::exists(cDir)) {
+            if (QFileInfo::exists(cDir)) {
                 themeDirs += cDir;
             }
         }
@@ -163,11 +193,11 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
         if (QDir(cDir).exists()) {
             themeDirs += cDir;
             if (d->mDir.isEmpty()) {
-                if (QFile::exists(cDir + "index.theme")) {
+                if (QFileInfo::exists(cDir + "index.theme")) {
                     d->mDir = cDir;
                     fileName = d->mDir + "index.theme";
                     mainSection = QStringLiteral("Icon Theme");
-                } else if (QFile::exists(cDir + "index.desktop")) {
+                } else if (QFileInfo::exists(cDir + "index.desktop")) {
                     d->mDir = cDir;
                     fileName = d->mDir + "index.desktop";
                     mainSection = QStringLiteral("KDE Icon Theme");
@@ -228,7 +258,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     groups += QStringLiteral("Small");
     groups += QStringLiteral("Panel");
     groups += QStringLiteral("Dialog");
-    const int defDefSizes[] = { 32, 22, 22, 16, 32, 32 };
+    const int defDefSizes[] = { 32, 22, 22, 16, 48, 32 };
     KConfigGroup cg(d->sharedConfig, mainSection);
     for (int i = 0; i < groups.size(); ++i) {
         const QString group = groups.at(i);
@@ -559,8 +589,8 @@ QStringList KIconTheme::list()
             if ((*it2).startsWith(QLatin1String("default."))) {
                 continue;
             }
-            if (!QFile::exists(it + QLatin1Char('/') + *it2 + QLatin1String("/index.desktop")) &&
-                    !QFile::exists(it + QLatin1Char('/') + *it2 + QLatin1String("/index.theme"))) {
+            if (!QFileInfo::exists(it + QLatin1Char('/') + *it2 + QLatin1String("/index.desktop")) &&
+                    !QFileInfo::exists(it + QLatin1Char('/') + *it2 + QLatin1String("/index.theme"))) {
                 continue;
             }
             KIconTheme oink(*it2);
@@ -699,7 +729,7 @@ QString KIconThemeDir::iconPath(const QString &name) const
     }
 
     const QString file = constructFileName(name);
-    if (QFile::exists(file)) {
+    if (QFileInfo::exists(file)) {
         return KLocalizedString::localizedFilePath(file);
     }
 
